unit uTSimRes;

interface

uses
{$IFNDEF FPC}
  DateUtils,
{$ENDIF}
  Math,
  xMatDefs,
  uauxiliares,
  uConstantesSimSEE,
  SysUtils, Classes;

type
  TResumenActor = class
  public
    Clase: string;
    nombre, infoAd: string;
    constructor Create(const nombre, clase, infoAd: string);
  end;

  TDAOfResumenActor = array of TResumenActor;

  TEncabezadoSimRes = class
  public
    cnt_Lineas: integer;
    cnt_LineaPrimerCronica: integer;
    dtIniSim, FechaIni, FechaFin: TDateTime;
    nCronicas: integer;
    nPasos, nPostes, nActores, nCols: integer;
    durpos: TDAofNReal;
    durPaso_: NReal;
    invDurPaso: NReal;
    resumenActores: TDAOfResumenActor;
    actoresPorColumnas: TStringList;
    unidadesPorColumnas: TStringList;
    variablesPorColumnas: TStringList;
    iCronica, rSeed: integer;
    constructor Create(var f: textfile);
    procedure Free; virtual;

    // auxiliares
    procedure SplitNomVar(const s: string; var nomActor, nomVar, unidades: string);
    procedure JoinNomVar(const nomActor, nomVar: string; var s: string);
  private
    procedure readln(var f: textfile; var linea: string);
  end;

  { TResultadoSim }
  TResultadoSim = class
  public
    archiSimRes: string;
    f: textfile;
    iCronica, rSeed: integer;
    rec: TDAOfNReal;
    iCronica_base: integer;

    {$IFDEF pruebasDistSimRes3}
    procedure imprimir;
    {$ENDIF}

    constructor Create(const simres: string);

    procedure reset; virtual;
    procedure First; virtual;
    // lee el siguiente record
    procedure Next; virtual;
    function EOF: boolean; virtual;

    function nCols: integer;

    function nombreColumna(iColumna: integer): string;
    procedure nomActUniVarColumna(iColumna: integer;
      var nomActor, nomVar, unidades: string);

    //Retornan el numero de la columna identificada por nomActor, nomVar y unidades
    //La primer versin recibe el id joineado con JoinNomVar, la segunda recibe
    //los valores por separado
    function columnaActUniVar(const idActUniVar: string): integer; overload;
    function columnaActUniVar(const nomActor, nomVar: string): integer; overload;

    procedure Free; virtual;

    // retorna el nmero de paso correspondiente a una fecha
    // el nmero de paso va de 1 a NPasos.
    function kPasoInicioParaFecha(fecha: TDateTime): integer;
    function fechaDeInicioDelPaso(kPaso: integer): TDateTime;

    function suma(const indices: array of integer): NReal;
    function maximo(const indices: array of integer): NReal;
    procedure sumaConSigno(const indices: array of integer;
      var sumaPos, sumaNeg: NReal);
    function combinar(const indices: TMatOfNInt; coefs: TDAofNReal): NReal;
    function promedio(const indices: array of integer): NReal;

    // indices debe tener los indices de las columnas de la variable
    // para cada uno de los postes ordenados en forma creciente
    // retorna la suma producto de las variables por la duracin de los postes
    function sumaproductocondurpos(const indices: array of integer): NReal;

    procedure sumaproductocondurposConSigno(const indices: array of integer;
      var sumaPos, sumaNeg: NReal);

    // Similar a la anterior, pero solamente suma hasta el poste kPosteHasta inclusive
    // Por ejemplo si kposteHasta= 3 se realizar la suma
    // producto solamente para los postes 1, 2 y 3.
    // Esto sirve para tener la suma por ejemplo de las HORAS FIRMES que se
    // definen de acuerdo al reglamento como las horas fuera del valle.
    function sumaproductoconDurposHasta(const indices: array of integer;
      kPosteHasta: integer): NReal;

    // indices debe tener los indices de las columnas de la variable
    // para cada uno de los postes ordenados en forma creciente
    // retorna el promedio ponderado de las variables por la duracin de los postes
    // (v1 * durpos[1] + v2 * durpos[2] + ... + vn * durpos[n]) / sum(durpos)
    function promedioPonderadoPorDurpos(const indices: array of integer): NReal;

    // sum( min(A[k],TopeDeA)*DurPos[k] )
    // en recorte devuelve la suma de los recortes de aplicar el tope
    function sumaProductoConDurposTopeado(const indicesA: array of integer;
      TopeDeA: NReal; var Recorte: NReal): NReal;

    // sum( min(A[k],TopeDeA)*B[k]*DurPos[k] )  til para clculo de ingresos
    // en recorte devuelve la suma de los recortes de aplicar el tope
    function sumaDobleProductoConDurposTopeado(
      const indicesA, indicesB: array of integer; TopeDeB: NReal;
      var Recorte: NReal): NReal;

    // operaciones multitramo
    function suma_m(const indices: array of TDAOfNInt): NReal;
    function promedio_m(const indices: array of TDAOfNInt): NReal;
    function sumaproductocondurpos_m(const indices: array of TDAOfNInt): NReal;
    function maximo_m(const indices: array of TDAOfNInt): NReal;

    // busca el actor y la variable y retorna el array de ndices
    // correspondientes a los postes 1, .... NPostes
    function getIndices(const actor, variable: string): TDAOfNInt;
    function ncronicas: integer; virtual;
    function npasos: integer; virtual;
    function durpaso_: NReal; virtual;

  private
    encabezado_: TEncabezadoSimRes;
    r_next: string;
    procedure readln(var f: textfile; var linea: string);
  end;



  { TResultadosSim_MultiARchivo }

  TResultadosSim_MultiARchivo = class(TResultadoSim)
  public
    arr_archiSimRes: TDAOfString;
    arr_encabezados: array of TEncabezadoSimRes;
    nCronicasTotales: integer;


    iFile: integer; // archivo activo.


    constructor Create(const simres: TDAOfString);

    procedure reset; override;
    procedure First; override;
    // lee el siguiente record
    procedure Next; override;
    function EOF: boolean; override;
    procedure Free; override;
    function ncronicas: integer; override;
    function npasos: integer; override;
    function durpaso_: NReal; override;
  end;

        (*
  TResultadoSimFiltradoYBuffereado = class(TResultadoSim)
    public
      buffer: array of TDAOfNReal;
      fechaDesde, fechaHasta: TDateTime;
      kPasoDesde, kPasoHasta: integer;
      kBuffRead: integer;
      nCronicas, nPasosDelFiltro: integer;

      Constructor Create(simres : String; fechaDesde, fechaHasta: TDateTime);
      procedure first; override;
      procedure Next; override;
      function EOF: boolean; override;
      procedure Free; override;

  end;

      *)
//Retorna el texto que haya hasta el proximo tab o el final de linea
function nextTab(var s: string): string;

implementation

function IsoStrToDateTime(const s: string): TDateTime;
var
  //  anio, mes, dia, hora: string;
  anio, mes, dia, hora, minuto: integer;
begin
  //  s[11]:= ' ';
(*
  anio:= copy( s, 1, 4 );
  mes:= copy( s, 6,2 );
  dia:= copy( s, 9, 2 );
  hora:= copy( s, 12, 5 );
*)

  anio := StrToInt(copy(s, 1, 4));
  mes := StrToInt(copy(s, 6, 2));
  dia := StrToInt(copy(s, 9, 2));
  if (length(s) > 12) then
  begin
    hora := StrToInt(copy(s, 12, 2));
    minuto := StrToInt(copy(s, 15, 2));
  end
  else
  begin
    hora := 0;
    minuto := 0;
  end;
{$IFDEF FPC}
  Result := ComposeDateTime(EncodeDate(anio, mes, dia), EncodeTime(
    hora, minuto, 0, 0));
{$ELSE}
  Result := EncodeDateTime(anio, mes, dia, hora, minuto, 0, 0);
{$ENDIF}
end;


procedure TEncabezadoSimRes.readln(var f: textfile; var linea: string);
begin
  system.readln(f, linea);
  Inc(cnt_Lineas);
end;


procedure TEncabezadoSimRes.SplitNomVar(const s: string;
  var nomActor, nomVar, unidades: string);
var
  posComa: integer;
begin
  posComa := Pos('.', s);
  nomActor := Copy(s, 0, posComa - 1);
  nomVar := Copy(s, posComa + 2, Length(s) - posComa + 2);
  posComa := Pos('[', nomVar);
  unidades := Copy(nomVar, posComa + 2, length(nomVar) - posComa + 2);
  nomVar := copy(nomVar, 0, posComa - 1);
end;

procedure TEncabezadoSimRes.JoinNomVar(const nomActor, nomVar: string; var s: string);
begin
  s := nomActor + '_' + nomVar;
end;

constructor TEncabezadoSimRes.Create(var f: textfile);
var
  i: integer;
  linea, nomActor, tipoActor, infoActor: string;
  encabActores: string;
  encabUnidades: string;
  encabVariables: string;
  unidades_str: string;
begin
  cnt_Lineas := 0;
  //Version simulador
  Readln(f, linea);
  //Inicio simulacin:
  //Los archivos viejos no incluan la versin del simulador
  if pos('Versin del simulador:', linea) = 1 then
    Readln(f, linea);
  nextTab(linea);
  dtIniSim := IsoStrToDateTime(linea);

  //FechaIni FechaFin
  Readln(f, linea);
  nextTab(linea);
  FechaIni := IsoStrToDateTime(nextTab(linea));
  nextTab(linea);
  FechaFin := IsoStrToDateTime(nextTab(linea));
  //NCronicas
  Readln(f, linea);
  nexttab(linea);
  nCronicas := NextInt(linea);
  //NPasos
  Readln(f, linea);
  nexttab(linea);
  nPasos := NextInt(linea);
  //NPostes
  Readln(f, linea);
  nexttab(linea);
  nPostes := NextInt(linea);
  SetLength(durPos, nPostes);
  //Durpos
  Readln(f, linea);
  nexttab(linea);
  for i := 0 to nPostes - 1 do
    durPos[i] := NextFloat(linea);
  durPaso_ := vsum(durPos);
  invDurPaso := 1 / durPaso_;
  //NActores
  Readln(f, linea);
  nexttab(linea);
  nActores := NextInt(linea);
  SetLength(resumenActores, nActores);
  readln(f, linea);
  for i := 0 to nActores - 1 do
  begin
    Readln(f, linea);
    nomActor := nextTab(linea);
    tipoActor := nextTab(linea);
    infoActor := nextTab(linea);
    resumenActores[i] := TResumenActor.Create(nomActor, tipoActor, infoActor);
  end;

  (********* INICIO PRIMERA CRONICA ******)
  cnt_LineaPrimerCronica := cnt_Lineas + 1;

  readln(f, linea);
  //Cronica RandSeed
  Readln(f, linea);
  nexttab(linea);
  iCronica := nextInt(linea);
  nexttab(linea);
  rSeed := nextInt(linea);

  //Actores
  Readln(f, encabActores);
  linea := Copy(encabActores, 0, length(encabActores));

  actoresPorColumnas := TStringList.Create;
  nCols := 0;
  while linea <> '' do
  begin
    Inc(nCols);
    actoresPorColumnas.Add(nextTab(linea));
  end;

  //Unidades
  readln(f, encabUnidades);
  linea := Copy(encabUnidades, 0, length(encabUnidades));
  unidadesPorColumnas := TStringList.Create;
  unidadesPorColumnas.Capacity := actoresPorColumnas.Count;

  while linea <> '' do
  begin
    unidades_str := nextTab(linea);
    if pos('[', unidades_str) = 0 then
      unidades_str := '[' + unidades_str + ']';
    unidadesPorColumnas.Add(unidades_str);
  end;

  //Variables
  readln(f, encabVariables);
  linea := Copy(encabVariables, 0, length(encabVariables));
  variablesPorColumnas := TStringList.Create;
  variablesPorColumnas.Capacity := actoresPorColumnas.Count;

  while linea <> '' do
    variablesPorColumnas.Add(nextTab(linea));

  //Postes
  readln(f, linea);
end;


procedure TEncabezadoSimRes.Free;
var
  k: integer;
begin
  setlength(durpos, 0);
  for k := 0 to high(resumenActores) do
    resumenActores[k].Free;
  setlength(resumenActores, 0);
  actoresPorColumnas.Free;
  unidadesPorColumnas.Free;
  variablesPorColumnas.Free;
  inherited Free;
end;
//------------------------
//Mtodos de TResumenActor
//========================
constructor TResumenActor.Create(const nombre, clase, infoAd: string);
begin
  inherited Create();
  self.nombre := nombre;
  self.infoAd := infoAd;
  self.Clase := clase;
end;

//--------------------------
//Mtodos de DTResultadoSim
//==========================

procedure TResultadoSim.readln(var f: textfile; var linea: string);
begin
  linea := r_next;
  system.readln(f, r_next);
end;


function TResultadoSim.kPasoInicioParaFecha(fecha: TDateTime): integer;
var
  df: double;
begin
  if fecha <= encabezado_.FechaIni then
    Result := 1
  else if fecha >= encabezado_.FechaFin then
    Result := encabezado_.nPasos + 1
  else
  begin
    df := fecha - encabezado_.FechaIni;
    Result := trunc(df / (encabezado_.durPaso_ / 24.0) + 0.0001 ) + 1;
  end;
end;

function TResultadoSim.fechaDeInicioDelPaso(kPaso: integer): TDateTime;
begin
  Result := encabezado_.FechaIni + (kPaso - 1) * encabezado_.durPaso_ / 24.0;
end;

{$IFDEF pruebasDistSimRes3}
procedure TResultadoSim.imprimir;
var
  i: integer;
begin
  writeln(archiSimRes);
  for i := 0 to 10 do
    Write(rec[i], ' ');

  writeln;
end;

{$ENDIF}

constructor TResultadoSim.Create(const simres: string);
var
  oldFileMode: integer;
  ts: string;
begin
  {$IFDEF MULTI_ARCHIVO}
  writeln(simres);
  {$ENDIF}
  if FileExists(simres) { *Converted from FileExists*  } then
  begin

    inherited Create;
    archiSimRes := simres;
    assignfile(f, simres);

    oldFileMode := filemode;
    filemode := 0;
    system.reset(f);
    filemode := oldfilemode;

    encabezado_ := TEncabezadoSimRes.Create(f);
    setlength(rec, encabezado_.nCols);
    iCronica := encabezado_.iCronica;
    rSeed := encabezado_.rSeed;

    r_next := '';
    readln(f, ts); // solo para llenar r_next
  end
  else
    raise Exception.Create('TResultadoSim.Create: error, no se encuentra el archivo '
      + simres);
end;

procedure TResultadoSim.reset;
var
  k: integer;
  r: string;
begin
  system.reset(f);
  for k := 1 to encabezado_.cnt_LineaPrimerCronica do
    readln(f, r);
  //iCronica:= encabezado_.iCronica;
  //rSeed:= encabezado_.rSeed;
  Next;
end;

procedure TResultadoSim.First;
begin
  reset;
  Next;
end;

function TResultadoSim.EOF: boolean;
begin
  Result := system.EOF(f);
end;

procedure TResultadoSim.Next;
var
  linea: string;
  k: integer;
begin
  readln(f, linea);

  if linea = '' then
  begin
    readln(f, linea);
    nexttab(linea);
    iCronica := nextInt(linea) + iCronica_base;
    writeln('i ' + IntToStr(iCronica - iCronica_base) + ' ibase ' +
      IntToStr(iCronica_base));
    nexttab(linea);
    rSeed := nextInt(linea);

    readln(f, linea);
    readln(f, linea);
    readln(f, linea);
    readln(f, linea);
    readln(f, linea);
  end;

  //writeln(copy( linea,0,15 ));

  rec[0] := NextInt(linea);
  rec[1] := IsoStrToDateTime(nexttab(linea));
  for k := 2 to encabezado_.nCols - 1 do
    rec[k] := nextFloat(linea);
end;


function TResultadoSim.nCols: integer;
begin
  Result := encabezado_.nCols;
end;


function TResultadoSim.nombreColumna(iColumna: integer): string;
var
  res: string;
begin
  encabezado_.JoinNomVar(encabezado_.actoresPorColumnas[iColumna],
    encabezado_.VariablesPorColumnas[iColumna],
    res);
  Result := res;
end;

procedure TResultadoSim.nomActUniVarColumna(iColumna: integer;
  var nomActor, nomVar, unidades: string);
begin
  nomActor := encabezado_.ActoresPorColumnas[iColumna];
  nomVar := encabezado_.VariablesPorColumnas[iColumna];
  unidades := encabezado_.UnidadesPorColumnas[iColumna];
end;

function TResultadoSim.columnaActUniVar(const idActUniVar: string): integer;
var
  nomAct, nomVar, unidades: string;
begin
  encabezado_.SplitNomVar(idActUniVar, nomAct, nomVar, unidades);
  Result := columnaActUniVar(nomAct, nomVar);
end;

function TResultadoSim.columnaActUniVar(const nomActor, nomVar: string): integer;
var
  res, i: integer;
begin
  res := -1;
  for i := 0 to encabezado_.actoresPorColumnas.Count - 1 do
    if (((encabezado_.actoresPorColumnas[i] = nomActor) or
      ((nomActor = '-') and (encabezado_.actoresPorColumnas[i] = 'Sala'))) and
      (encabezado_.variablesPorColumnas[i] = nomVar)) then
    begin
      res := i;
      break;
    end;
  Result := res;
end;

function TResultadoSim.getIndices(const actor, variable: string): TDAOfNInt;
var
  kPoste: integer;
  res: TDAOfNInt;
  svar: string;
begin
  setlength(res, encabezado_.nPostes);
  for kPoste := 0 to encabezado_.nPostes - 1 do
  begin
    svar := variable + '_P' + IntToStr(kPoste + 1);
    res[kPoste] := columnaActUniVar(actor, svar);
    if res[kPoste] < 0 then
    begin
      setlength(res, 1);
      res[0] := columnaActUniVar(actor, variable);
      if res[0] < 0 then
        if variable <> 'CFaux' then
          raise Exception.Create('No encontr variable: ' +
            variable + ' del actor: ' + actor)
        else
          raise Exception.Create('No encontr variable: ' +
            variable + ' del actor: ' + actor +
            '. Asegurese de haber ejecutado la simulacin seleccionando un Costo Futuro Auxiliar.');
      break;
    end;
  end;
  Result := res;
end;


function TResultadoSim.ncronicas: integer;
begin
  Result := encabezado_.nCronicas;
end;

function TResultadoSim.npasos: integer;
begin
  Result := encabezado_.nPasos;
end;

function TResultadoSim.durpaso_: NReal;
begin
  Result := encabezado_.durpaso_;
end;


function TResultadoSim.promedio(const indices: array of integer): NReal;
var
  res: NReal;
  k: integer;
begin
  res := 0;
  for k := low(indices) to high(indices) do
    res := res + self.rec[indices[k]];
  Result := res / length(indices);
end;

function TResultadoSim.promedio_m(const indices: array of TDAOfNInt): NReal;
var
  kparte: integer;
  res: NReal;
  largoTotal: integer;
begin
  res := 0;
  largoTotal := 0;
  for kparte := 0 to high(indices) do
  begin
    largoTotal := largoTotal + length(indices[kparte]);
    res := res + suma(indices[kparte]);
  end;
  Result := res / largoTotal;
end;


function TResultadoSim.suma(const indices: array of integer): NReal;
var
  res: NReal;
  k: integer;
begin
  res := 0;
  for k := low(indices) to high(indices) do
    res := res + self.rec[indices[k]];
  Result := res;
end;

function TResultadoSim.maximo(const indices: array of integer): NReal;
var
  res: NReal;
  k: integer;
begin
  res := -MaxNReal;
  for k := low(indices) to high(indices) do
    if self.rec[indices[k]] > res then
      res := self.rec[indices[k]];
  Result := res;
end;

procedure TResultadoSim.sumaConSigno(const indices: array of integer;
  var sumaPos, sumaNeg: NReal);
var
  k: integer;
  v: NReal;
begin
  sumaPos := 0;
  sumaNeg := 0;
  for k := low(indices) to high(indices) do
  begin
    v := self.rec[indices[k]];
    if v >= 0 then
      sumaPos := sumaPos + v
    else
      sumaNeg := sumaNeg + v;
  end;
end;

function TResultadoSim.combinar(const indices: TMatOfNInt; coefs: TDAofNReal): NReal;
var
  res: NReal;
  k, j: integer;
begin
  res := 0;
  for k := low(indices) to high(indices) do
    for j := low(indices[k]) to high(indices[k]) do
      res := res + self.rec[indices[k][j]] * coefs[k];
  Result := res;
end;

function TResultadoSim.suma_m(const indices: array of TDAOfNInt): NReal;
var
  kparte: integer;
  res: NReal;
begin
  res := 0;
  for kparte := 0 to high(indices) do
    res := res + suma(indices[kparte]);
  Result := res;
end;

function TResultadoSim.sumaproductocondurpos(const indices: array of integer): NReal;
var
  res: NReal;
  kPoste: integer;
  kI: integer;
begin
  res := 0;
  for kPoste := 0 to encabezado_.nPostes - 1 do
  begin
    kI := min(kPoste, high(indices));
    res := res + rec[indices[kI]] * encabezado_.durpos[kPoste];
  end;
  Result := res;
end;

procedure TResultadoSim.sumaproductocondurposConSigno(const indices: array of integer;
  var sumaPos, sumaNeg: NReal);
var
  kPoste, kI: integer;
  v: NReal;
begin
  sumaPos := 0;
  sumaNeg := 0;
  for kPoste := 0 to encabezado_.nPostes - 1 do
  begin
    kI := min(kPoste, high(indices));
    v := rec[indices[kI]] * encabezado_.durpos[kPoste];
    if v >= 0 then
      sumaPos := sumaPos + v
    else
      sumaNeg := sumaNeg + v;
  end;
end;

function TResultadoSim.sumaproductoconDurposHasta(const indices: array of integer;
  kposteHasta: integer): NReal;
var
  res: NReal;
  kPoste: integer;
  kI: integer;
begin
  res := 0;
  for kPoste := 0 to kposteHasta - 1 do
  begin
    kI := min(kPoste, high(indices));
    res := res + rec[indices[kI]] * encabezado_.durpos[kPoste];
  end;
  Result := res;
end;

function TResultadoSim.promedioPonderadoPorDurpos(
  const indices: array of integer): NReal;
var
  res: NReal;
  kPoste, kI: integer;
begin
  res := 0;
  for kPoste := 0 to encabezado_.nPostes - 1 do
  begin
    kI := min(kPoste, high(indices));
    res := res + rec[indices[kI]] * encabezado_.durpos[kPoste];
  end;

  Result := res * encabezado_.invDurPaso;
end;

// sum( min(A[k],TopeDeA)*DurPos[k] )
// en recorte devuelve la suma de los recortes de aplicar el tope
function TResultadoSim.sumaProductoConDurposTopeado(const indicesA: array of integer;
  TopeDeA: NReal; var Recorte: NReal): NReal;
var
  res: NReal;
  kPoste, kA: integer;
  a: NReal;
begin
  res := 0;
  recorte := 0;
  for kPoste := 0 to encabezado_.nPostes - 1 do
  begin
    kA := min(kPoste, high(indicesA));
    a := self.rec[indicesA[kA]];
    if a <= TopeDeA then
    begin
      res := res + a * encabezado_.durpos[kPoste];
    end
    else
    begin
      res := res + TopeDeA * encabezado_.durpos[kPoste];
      recorte := recorte + (a - TopeDeA) * encabezado_.durpos[kPoste];
    end;
  end;
  Result := res;
end;

function TResultadoSim.sumaDobleProductoConDurposTopeado(
  const indicesA, indicesB: array of integer; TopeDeB: NReal; var Recorte: NReal): NReal;
var
  res: NReal;
  kPoste: integer;
  valB: NReal;
  kA, kB: integer;
begin
  res := 0;
  recorte := 0;
  for kPoste := 0 to encabezado_.nPostes - 1 do
  begin
    kA := min(kPoste, high(indicesA));
    kB := min(kPoste, high(indicesB));

    valB := self.rec[indicesB[kB]];
    if valB > TopeDeB then
    begin
      res := res + self.rec[indicesA[kA]] * TopeDeB * encabezado_.durpos[kPoste];
      recorte := recorte + self.rec[indicesA[kA]] * (valB - TopeDeB) *
        encabezado_.durpos[kPoste];
    end
    else
      res := res + self.rec[indicesA[kA]] * valB * encabezado_.durpos[kPoste];
  end;
  Result := res;
end;

function TResultadoSim.sumaproductocondurpos_m(
  const indices: array of TDAOfNInt): NReal;
var
  kIndice: integer;
  res: NReal;
begin
  res := 0;
  for kIndice := 0 to high(indices) do
    res := res + sumaproductocondurpos(indices[kIndice]);
  Result := res;
end;

function TResultadoSim.maximo_m(const indices: array of TDAOfNInt): NReal;
var
  i: integer;
  res, maxI: NReal;
begin
  res := -MaxNReal;
  for i := 0 to High(indices) do
  begin
    maxI := maximo(indices[i]);
    if maxI > res then
      res := maxI;
  end;
  Result := res;
end;

procedure TResultadoSim.Free;
begin
  setlength(rec, 0);
  //encabezado_.Free;
  closefile(f);
  inherited Free;
end;

function nextTab(var s: string): string;
var
  posTab: integer;
  res: string;
begin
  posTab := Pos(#9, s);
  if posTab <> 0 then
  begin
    res := Copy(s, 1, posTab - 1);
    Delete(s, 1, posTab);
  end
  else
  begin
    res := copy(s, 1, length(s));
    s := '';
  end;
  Result := trim(res);
end;

{ rch@201309292254
Si el archivo existe retorna TRUE, si no existe se fija
si tiene sin sustituir el $escenario y si es as busca si
existe un archivo con el mismo nombre de la sala y con _Base y otro
}

function Test_FileExists(var archi: string): boolean;
var
  Info: TSearchRec;
  Count: longint;
  ip, ip2: integer;
  s: string;
  old_dir: string;
  archi_dir: string;
  archi_name: string;
  archi_encontrado: string;
  time_encontrado: longint;

begin
  if fileExists(archi) then
  begin
    Result := True;
    exit;
  end;

  ip := pos('{$escenario}', archi);
  if ip = 0 then
  begin
    Result := False;
    exit;
  end;

  old_dir := getcurrentdir;

  archi_dir := ExtractFileDir(archi);
  archi_name := ExtractFileName(archi);
  setcurrentdir(archi_dir);

  ip := pos('{$escenario}', archi_name);
  ip2 := ip + length('{$escenario}');
  s := copy(archi_name, 1, ip - 1) + '*' + copy(archi_name, ip2, length(archi_name) - ip2 + 1);

  Count := 0;
  if FindFirst(s, faAnyFile, Info) = 0 then
  begin
    repeat
      Inc(Count);
      if ( count = 1) OR ( Info.Time > time_encontrado ) then
      begin
        archi_encontrado := info.Name;
        time_encontrado := info.time;
      end;
    until FindNext(info) <> 0;
  end;
  FindClose(Info);
  setcurrentdir( old_dir );
  if count = 0 then
    result:= false
  else
  begin
    archi:= archi_dir+ barraDir + archi_encontrado;
    result:= true;
  end;
end;

constructor TResultadosSim_MultiARchivo.Create(const simres: TDAOfString);
var
  k: integer;
  oldFileMode: byte;

begin
  //La idea es tener en un array todos los encabezados, para eso abro cada
  //archivo y cargo el encabezado al array.

  arr_archiSimRes := simres;
  setlength(arr_encabezados, length(arr_archiSimRes));
  nCronicasTotales := 0;
  for k := 0 to high(simres) do
  begin
    if Test_FileExists(simres[k]) then
    begin
      assignfile(f, simres[k]);
      oldFileMode := filemode;
      filemode := 0;
      system.reset(f);
      filemode := oldfilemode;
      arr_encabezados[k] := TEncabezadoSimRes.Create(f);
      //Acumulo la cantidad de cronicas por archivo.
      nCronicasTotales := nCronicasTotales + arr_encabezados[k].nCronicas;
      Close(f);
    end
    else
      raise Exception.Create(
        'TResultadosSim_MultiARchivo.Create: error, no se encuentra el archivo ' +
        simres[k]);
  end;

  iCronica_base := 0;
  iFile := 0;
  //Creo el resultadoSim del primer archivo de la lista.
  inherited Create(arr_archiSimRes[0]);
end;

procedure TResultadosSim_MultiARchivo.reset;
begin
  if iFile > 0 then
  begin
    closefile(f);
    Assign(f, arr_archiSimRes[0]);
    iFile := 0;
  end;
  iCronica_base := 0;
  inherited reset;
end;

procedure TResultadosSim_MultiARchivo.First;
begin
  inherited First;
end;

// lee el siguiente record
procedure TResultadosSim_MultiARchivo.Next;
begin
  if inherited EOF then
  begin
    if (iFile < high(arr_archiSimRes)) then
    begin
      encabezado_ := arr_encabezados[iFile];

      iCronica_base := iCronica_base + encabezado_.nCronicas;
      closefile(f);
      Inc(iFile);

      writeln('file ' + IntToStr(iFile));

      Assign(f, arr_archiSiMRes[iFile]);
      inherited reset;
    end;
  end
  else
    inherited Next;
end;

function TResultadosSim_MultiARchivo.EOF: boolean;
begin
  Result := (iFile >= high(arr_archiSimRes)) and inherited EOF;
end;

procedure TResultadosSim_MultiARchivo.Free;
var
  k: integer;
begin
  for k := 0 to high(arr_encabezados) do
    arr_encabezados[k].Free;
  setlength(arr_encabezados, 0);
  inherited Free;
end;

function TResultadosSim_MultiARchivo.ncronicas: integer;
begin
  Result := nCronicasTotales;
end;

function TResultadosSim_MultiARchivo.npasos: integer;
begin
  Result := arr_encabezados[iFile].npasos;
end;


function TResultadosSim_MultiARchivo.durpaso_: NReal;
begin
  Result := arr_encabezados[iFile].durpaso_;
end;


(*

Constructor TResultadoSimFiltradoYBuffereado.Create(simres : String; fechaDesde, fechaHasta: TDateTime);
var
  k: integer;
  iPaso, kPaso: integer;
begin
  inherited Create( simres );
  self.fechaDesde:= fechaDesde;
  self.fechaHasta:= fechaHasta;

  kPasoDesde:= kPasoInicioParaFecha( fechaDesde );
  kPasoHasta:= kPasoInicioParaFecha( fechaHasta );

  nPasosDelFiltro:= kPasoHasta-kPasoDesde;
  nCronicas:= encabezado.nCronicas;
  setlength( buffer, nCronicas * nPasosDelFiltro );

  iRec:= 0;
  while not inherited EOF do
  begin
    inherited next;
    kPaso:= round(rec[0]);
    if ( kPaso < kPasoDesde ) or ( kPaso >= kPasoHasta) then continue;
    iPaso:= kPaso - kPasoDesde;
    buffer[ iRec ]:= rec;
    inc( iRec );
    setlength( rec, encabezado.ncols );
  end;
  iRec:= 0;
  iCronica:= 1;



end;

procedure TResultadoSimFiltradoYBuffereado.first; override;
procedure TResultadoSimFiltradoYBuffereado.Next; override;
function TResultadoSimFiltradoYBuffereado.EOF: boolean; override;
procedure TResultadoSimFiltradoYBuffereado.Free; override;
end;

    *)
end.
